package dbfit.fixture;

import java.sql.Connection;

import org.dbunit.database.DatabaseConnection;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.ReplacementDataSet;
import org.dbunit.dataset.excel.XlsDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSet;
import org.dbunit.operation.DatabaseOperation;

import dbfit.environment.DBEnvironment;
import dbfit.environment.DbEnvironmentFactory;
import dbfit.util.FileHelper;
import dbfit.util.Log;
import fit.ColumnFixture;

/**
 * Fixture for integrating FitNesse with dbUnit.
 * 
 * @author Bjrn Beskow, Callista Enterprise AB
 * @author Pl Brattberg, Acando AB
 * @since 1.2
 */
public class TestData extends ColumnFixture {
	private DBEnvironment dbEnvironment;
	private String fileType;
	public String operation;
	public String file;

	public TestData() {
		dbEnvironment = DbEnvironmentFactory.getDefaultEnvironment();
	}

	public TestData(DBEnvironment environment, String fileType) {
		this.dbEnvironment = environment;
		this.fileType = fileType;
		System.out.println("Filetype: " + fileType);
	}

	/**
	 * Perform a DbUnit operation on a dataset from an external file.
	 */
	@Override
	public void execute() throws Exception {
		IDataSet dataSet = null;
		if ("xls".equalsIgnoreCase(fileType)) {
			dataSet = new XlsDataSet(FileHelper.getFileAsStream(file));
		} else if ("xml".equalsIgnoreCase(fileType)) {
			dataSet = new FlatXmlDataSet(FileHelper.getFileAsStream(file));
		} else {
			throw new IllegalArgumentException(String.format(
					"Cannot handle files of type '%s'. "
							+ "Use one of 'xls' or 'xml'.", fileType));
		}
		performOperation(operation, dataSet);
	}

	protected void performOperation(String operation, IDataSet dataSet)
			throws Exception {
		performOperation(getDbUnitOperation(operation), dataSet);
	}

	protected void performOperation(DatabaseOperation operation,
			IDataSet dataSet) throws Exception {
		Log.logf("Running operation %s with content from file %s\n", operation,
				file);
		ReplacementDataSet replacement = new ReplacementDataSet(dataSet);
		replacement.addReplacementObject("null", null);
		operation.execute(getConnection(dbEnvironment.getConnection()),
				replacement);
	}

	protected DatabaseOperation getDbUnitOperation(String operation) {
		String upcaseOperation = operation.toUpperCase();
		if ("CLEAN_INSERT".equals(upcaseOperation)) {
			return DatabaseOperation.CLEAN_INSERT;
		} else if ("INSERT".equals(upcaseOperation)) {
			return DatabaseOperation.INSERT;
		} else if ("UPDATE".equals(upcaseOperation)) {
			return DatabaseOperation.UPDATE;
		} else if ("REFRESH".equals(upcaseOperation)) {
			return DatabaseOperation.REFRESH;
		} else if ("DELETE".equals(upcaseOperation)) {
			return DatabaseOperation.DELETE;
		} else if ("DELETE_ALL".equals(upcaseOperation)) {
			return DatabaseOperation.DELETE_ALL;
		} else if ("TRUNCATE_TABLE".equals(upcaseOperation)) {
			return DatabaseOperation.TRUNCATE_TABLE;
		} else {
			throw new IllegalArgumentException("Invalid operation " + operation);
		}
	}

	protected IDatabaseConnection getConnection(Connection jdbcConnection)
			throws Exception {
		IDatabaseConnection connection = new DatabaseConnection(jdbcConnection);
		return connection;
	}
}
